@@ -23,6 +23,8 @@
 #include <glib.h>
 #include <glib-object.h>
 
+#include <pthread.h>
+
 #define AGS_TYPE_TRACK                (ags_track_get_type())
 #define AGS_TRACK(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), AGS_TYPE_TRACK, AgsTrack))
 #define AGS_TRACK_CLASS(class)        (G_TYPE_CHECK_CLASS_CAST((class), AGS_TYPE_TRACK, AgsTrackClass))
@@ -50,7 +52,7 @@ typedef enum{
 
 struct _AgsTrack
 {
-  GObject object;
+  GObject gobject;
 
   guint flags;
 
@@ -65,7 +67,7 @@ struct _AgsTrack
 
 struct _AgsTrackClass
 {
-  GObjectClass object;
+  GObjectClass gobject;
 };
 
 GType ags_track_get_type();
